/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*   Files: main_fmc.cpp, findClique.cpp, findCliqueHeu.cpp, utils.cpp		  */
/*   			findClique.h, findCliqueHeuOld.cpp, Makefile              		  */
/*                                                                           */                                                                        
/*   Description:  a library for finding the maximum clique from a graph     */                                                   
/*                                                                           */
/*                                                                           */
/*   Authors:  Md. Mostofa Ali Patwary and Bharath Pattabiraman              */
/*            EECS Department, Northwestern University                       */
/*            email: {mpatwary,bpa342}@eecs.northwestern.edu                 */
/*                                                                           */
/*   Copyright, 2012, Md. Mostofa Ali Patwary and Bharath Pattabiraman       */
/*   See COPYRIGHT notice in top-level directory.                            */
/*                                                                           */
/*   Please site the following publication if you use this package           */
/*	  Bharath Pattabiraman, Md. Mostofa Ali Patwary, Assefaw H. Gebremedhin2, */
/*	  Wei-keng Liao, and Alok Choudhary.												  */
/*	  "A Hierarchical Pruning Algorithm for											  */ 
/*	  the Maximum Clique Problem on Large Graphs"									  */
/*   submitted to ALENEX'13 																  */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "findClique.h"

//int *bitVec;

void maxCliqueHelper( CGraphIO& gio, vector<int>* U, int sizeOfClique, int& maxClq, vector<int>& max_clique_data_inter, int& pruned3)
{
	int iPos, index = 0, k, maxClq_prev;
	vector <long long>* ptrVertex = gio.GetVerticesPtr();
	vector <int>* ptrEdge = gio.GetEdgesPtr();
	vector <int> U_new;
	U_new.reserve(gio.GetVertexCount());
	vector <int> X;
	
	if( U->size() == 0  )
	{
		if( sizeOfClique > maxClq )
		{
			maxClq = sizeOfClique;
			max_clique_data_inter.clear();
		}
		return;
	}

	k = 0;
	while( U->size() > 0 )
	{
		//Old Pruning 2
		if( sizeOfClique + U->size() <= maxClq )
			return;

		index = U->back();
		U->pop_back();

		// Loop over U.
		/*		
		for(int i = 0; i < U->size(); i++)
		{
			// Loop over neighbrs of v_index.
			for(long long j = (*ptrVertex)[index]; j < (*ptrVertex)[index + 1]; j++ )
				if( (*ptrEdge)[j] == (*U)[i] )
				{
					//if(bitVec[(*ptrEdge)[j]] != 1)
					if( (*ptrVertex)[(*U)[i]+1] - (*ptrVertex)[(*U)[i]] >=  maxClq )
						U_new.push_back( (*ptrEdge)[j]);
				}
		}
		*/
	
			
		for(long long j = (*ptrVertex)[index]; j < (*ptrVertex)[index + 1]; j++ )
		{
			if( (*ptrVertex)[(*ptrEdge)[j]+1] - (*ptrVertex)[(*ptrEdge)[j]] >=  maxClq )
			{
				for(int i = 0; i < U->size(); i++)
				{
					if( (*ptrEdge)[j] == (*U)[i] )
					{
						//if(bitVec[(*ptrEdge)[j]] != 1)
						//if( (*ptrVertex)[(*U)[i]+1] - (*ptrVertex)[(*U)[i]] >=  maxClq )
							U_new.push_back( (*ptrEdge)[j]);
					}
				}
			}
			else
				pruned3++;
		}
	
		maxClq_prev = maxClq;	
	
		maxCliqueHelper( gio, &U_new, sizeOfClique + 1, maxClq, max_clique_data_inter, pruned3);
		
		if(maxClq > maxClq_prev)
			max_clique_data_inter.push_back(index);

		//U_new.clear();
		U_new = X;
	}
}

int maxClique( CGraphIO& gio , int l_bound, int &pruned, vector<int>& max_clique_data, int upper_bound)
{
	vector <long long>* ptrVertex = gio.GetVerticesPtr();
	vector <int>* ptrEdge = gio.GetEdgesPtr();
	vector <int> U;
	int maxClq = l_bound;
	vector <int> X;
	U.reserve(gio.GetVertexCount());
	//max_clique_data.clear();
	//max_clique_data.reserve(gio.GetVertexCount());
	
	vector<int> max_clique_data_inter;
	max_clique_data_inter.reserve(gio.GetVertexCount());
	max_clique_data.reserve(gio.GetVertexCount());

	cout << "Computing Max Clique... with lower bound " << maxClq << endl;
	pruned = 0;
	int pruned3 = 0, pruned6 = 0;

	int pruned2 = 0, tmp = 0;

	//Bit Vector to track if vertex has been considered previously.
	int *bitVec = new int[gio.GetVertexCount()];
	//bitVec = new int[gio.GetVertexCount()];
	memset(bitVec, 0, gio.GetVertexCount() * sizeof(int));

	/*
	#ifdef _DEBUG
	int progress = 0, one_percent = (int)(gio.GetVertexCount()/100);
	cout << "Progress in percent: ";
	if(one_percent <= 0)
		one_percent = 1;
	#endif
	*/

	int prev_maxClq;
	bool found_upper_bound = false;
	//for(int i = 0; i < gio.GetVertexCount(); i++)
	for(int i = gio.GetVertexCount()-1; i >= 0; i--)
	{
		if(found_upper_bound == true)
		{
			pruned6++;
			continue;
		}
		/*
		#ifdef _DEBUG
		progress++;
		if(progress % one_percent == 0)
			cout << (int)(progress/one_percent) << " ";
		#endif
		*/

		bitVec[i] = 1;
		prev_maxClq = maxClq;

		//U.clear();
		U = X;
		//New Pruning 1
		if( (*ptrVertex)[i+1] - (*ptrVertex)[i] < maxClq)
		{
			pruned++;
			continue;
		}

		//tmp = 0;
		for( long long j = (*ptrVertex)[i]; j < (*ptrVertex)[i + 1]; j++ )
		{	
			//if((*ptrEdge)[j] < i)
			//Old Pruning 1
			if(bitVec[(*ptrEdge)[j]] != 1)
				//New Pruning 2
				if( (*ptrVertex)[(*ptrEdge)[j]+1] - (*ptrVertex)[(*ptrEdge)[j]] >=  maxClq )			
				{
					//tmp++;
					U.push_back((*ptrEdge)[j]);
				} else {
					pruned2++;
				}
		}
		//pruned2 += ( (*ptrVertex)[i + 1] - (*ptrVertex)[i] ) - tmp;

		maxCliqueHelper( gio, &U, 1, maxClq, max_clique_data_inter, pruned3);

		if(maxClq > prev_maxClq)
		{
			max_clique_data_inter.push_back(i);
			max_clique_data = max_clique_data_inter;
			if(maxClq == upper_bound)
				found_upper_bound = true;
		}	
		
		//max_clique_data_inter.clear();
		max_clique_data_inter = X;
	}

	/*
	#ifdef _DEBUG
	cout << " 100" << endl;
	#endif
	*/

	delete [] bitVec;
	max_clique_data_inter.clear();

	#ifdef _DEBUG
	cout << "Pruned1 = " << pruned << endl;
	cout << "Pruned2 = " << pruned2 << endl;
	cout << "Pruned3 = " << pruned3 << endl;
	cout << "Pruned6 = " << pruned6 << endl;
	#endif

	return maxClq;
}

void print_max_clique(vector<int>& max_clique_data)
{
	cout << "The maximum clique: ";
	for(int i = 0; i < max_clique_data.size(); i++)
		cout << max_clique_data[i] + 1 << " ";
	cout << endl;
}
