/*   Files: main_fmc.cpp, findClique.cpp, findCliqueHeu.cpp, utils.cpp		  */
/*   			findClique.h, findCliqueHeuOld.cpp, Makefile              		  */
/*                                                                           */                                                                        
/*   Description:  a library for finding the maximum clique from a graph     */                                                   
/*                                                                           */
/*                                                                           */
/*   Authors:  Md. Mostofa Ali Patwary and Bharath Pattabiraman              */
/*            EECS Department, Northwestern University                       */
/*            email: {mpatwary,bpa342}@eecs.northwestern.edu                 */
/*                                                                           */
/*   Copyright, 2012, Md. Mostofa Ali Patwary and Bharath Pattabiraman       */
/*   See COPYRIGHT notice in top-level directory.                            */
/*                                                                           */
/*   Please site the following publication if you use this package           */
/*	  Bharath Pattabiraman, Md. Mostofa Ali Patwary, Assefaw H. Gebremedhin2, */
/*	  Wei-keng Liao, and Alok Choudhary.												  */
/*	  "A Hierarchical Pruning Algorithm for											  */ 
/*	  the Maximum Clique Problem on Large Graphs"									  */
/*   submitted to ALENEX'13 																  */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "findClique.h"

void maxCliqueHelperHeu(CGraphIO& gio, vector<int>* U, int sizeOfClique, int& maxClq, vector<int>& max_clique_data_inter)
{
	int iPos, index = 0, k, maxDegree, maxDegVtx, maxClq_prev;
	vector <long long>* ptrVertex = gio.GetVerticesPtr();
	vector <int>* ptrEdge = gio.GetEdgesPtr();
	vector <int> U_new;
	U_new.reserve(gio.GetVertexCount());

	if( U->size() == 0  )
	{
		if( sizeOfClique > maxClq )
		{
			maxClq = sizeOfClique;
			max_clique_data_inter.clear();
		}
		return;
	}

	maxDegree = 0;
	index = -1;

	//Getting the vertex in U with maximum degree.
	for(int i = 0; i < U->size(); i++)
		if( getDegree(ptrVertex, (*U)[i]) >= maxDegree )
			index = i;

	//Error checking
	if(index < 0)
	{
		cout << "Fatal Error" << endl;
		exit(-1);
	}

	//Adding the max degree vertex to the clq
	//vClq->push_back(maxDegVtx);

	//Removing max degree vertex from U
	maxDegVtx = (*U)[index];
/*
	(*U)[index] = U->back();
	U->pop_back();
*/
	// Computing over intersection of U and N(maxDegVtx).


	/*		
	for(int i = 0; i < U->size(); i++)
		for(long long j = (*ptrVertex)[maxDegVtx]; j < (*ptrVertex)[maxDegVtx + 1]; j++ )
			if( (*ptrEdge)[j] == (*U)[i] )
				if( (*ptrVertex)[(*U)[i]+1] - (*ptrVertex)[(*U)[i]] >=  maxClq )
					U_new.push_back( (*ptrEdge)[j]);
	
	*/


		
    for(long long j = (*ptrVertex)[maxDegVtx]; j < (*ptrVertex)[maxDegVtx + 1]; j++ )
	{
		if( (*ptrVertex)[(*ptrEdge)[j]+1] - (*ptrVertex)[(*ptrEdge)[j]] >=  maxClq )
		{
			for(int i = 0; i < U->size(); i++)
			{
				if( (*ptrEdge)[j] == (*U)[i] )
				{
					//if( (*ptrVertex)[(*U)[i]+1] - (*ptrVertex)[(*U)[i]] >=  maxClq )
						U_new.push_back( (*ptrEdge)[j]);
				}
			}
		}
	}

	maxClq_prev = maxClq;

	maxCliqueHelperHeu( gio, &U_new, sizeOfClique + 1, maxClq, max_clique_data_inter);
	
	if(maxClq > maxClq_prev)
		max_clique_data_inter.push_back((*U)[index]);
		//max_clique_data_inter.push_back(index);

	U_new.clear();
	U->clear();
}

int maxCliqueHeu( CGraphIO& gio, int &pruned, vector<int>& max_clique_data, int upper_bound)
{

	vector <long long>* ptrVertex = gio.GetVerticesPtr();
	vector <int>* ptrEdge = gio.GetEdgesPtr();
	vector <int> U;
	vector <int> X;
	U.reserve(gio.GetVertexCount());


	int maxClq=0;
	//int pruned = 0;
	int maxClq_prev;

    //max_clique_data.clear();
    max_clique_data.reserve(gio.GetVertexCount());

    vector<int> max_clique_data_inter;
    max_clique_data_inter.reserve(gio.GetVertexCount());

    //Bit Vector to track if vertex has been considered previously.
    int *bitVec = new int[gio.GetVertexCount()];
    memset(bitVec, 0, gio.GetVertexCount() * sizeof(int));


	int found_upper_bound = false;
	//cout << "Starting main computation" << endl;

	for(int i = gio.GetVertexCount()-1; i >= 0; i--)
	{
		if(found_upper_bound == true)
		{
			continue;
		}
		//if(i % 1000000 == 0)
		//	cout << "Starting proccessing " << i << endl; 
		//U.clear();
		U = X; //
		maxClq_prev = maxClq;
		bitVec[i] = 1;

		//cout << "Whether i will be pruned " << i << endl;

		if( (*ptrVertex)[i+1] - (*ptrVertex)[i] < maxClq)
		{
			pruned++;
			continue;
		}

		//cout << "Getting neighbors " << i << endl;

		for( long long j = (*ptrVertex)[i]; j < (*ptrVertex)[i + 1]; j++ )
		{	
			/*cout << j << " ";
                	cout << (*ptrEdge)[j] << " ";
			cout << (*ptrVertex)[(*ptrEdge)[j]] << " ";
			cout << (*ptrVertex)[(*ptrEdge)[j]+1] << " ";
			cout << gio.GetVertexCount() << " ";
			cout << endl;

			if((*ptrEdge)[j] < 0 || (*ptrEdge)[j] >= gio.GetVertexCount())
		        	cout << "Invalid vertex " << (*ptrEdge)[j] << " range 0 to " << gio.GetVertexCount() << endl;

	                if((*ptrEdge)[j] + 1 < 0 || (*ptrEdge)[j] + 1 >= gio.GetVertexCount())
				cout << "Invalid vertex " << (*ptrEdge)[j] << " range 0 to " << gio.GetVertexCount() << endl;
			*/

			if(bitVec[(*ptrEdge)[j]] != 1)
			{
				if( (*ptrVertex)[(*ptrEdge)[j]+1] - (*ptrVertex)[(*ptrEdge)[j]] >=  maxClq )			
					U.push_back((*ptrEdge)[j]);
			}
		}

		//cout << "Calling the recursion " << i << endl;

		maxCliqueHelperHeu( gio, &U, 1 , maxClq, max_clique_data_inter);

		//cout << "Done the recursion " << i << endl;

		if(maxClq > maxClq_prev)
		{
			max_clique_data_inter.push_back(i);
			max_clique_data = max_clique_data_inter;
			if(maxClq == upper_bound)
				found_upper_bound = true;
		}	
		
		//max_clique_data_inter.clear();
		max_clique_data_inter = X;
	}

	delete [] bitVec;
	max_clique_data_inter.clear();

	return maxClq;
}
