/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*   Files: main_fmc.cpp, findClique.cpp, findCliqueHeu.cpp, utils.cpp		  */
/*   			findClique.h, findCliqueHeuOld.cpp, Makefile              		  */
/*                                                                           */                                                                        
/*   Description:  a library for finding the maximum clique from a graph     */                                                   
/*                                                                           */
/*                                                                           */
/*   Authors:  Md. Mostofa Ali Patwary and Bharath Pattabiraman              */
/*            EECS Department, Northwestern University                       */
/*            email: {mpatwary,bpa342}@eecs.northwestern.edu                 */
/*                                                                           */
/*   Copyright, 2012, Md. Mostofa Ali Patwary and Bharath Pattabiraman       */
/*   See COPYRIGHT notice in top-level directory.                            */
/*                                                                           */
/*   Please site the following publication if you use this package           */
/*	  Bharath Pattabiraman, Md. Mostofa Ali Patwary, Assefaw H. Gebremedhin2, */
/*	  Wei-keng Liao, and Alok Choudhary.												  */
/*	  "A Hierarchical Pruning Algorithm for											  */ 
/*	  the Maximum Clique Problem on Large Graphs"									  */
/*   submitted to ALENEX'13 																  */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "findClique.h"

bool fexists(const char *filename)
{
	ifstream ifile(filename);
	return ifile;
}

double wtime() // returns wall time in seconds
{
	timeval tv;
	gettimeofday(&tv, NULL);
	return (double)tv.tv_sec + (double)tv.tv_usec/1000000;
}

void usage(char *argv0)
{
	const char *params =
		"Usage: %s -t algtype -i inputfile -s intial_clique_size -p -u upper_bound_clique_size\n"
		"	-t algorithm_type			: 0 for exact, 1 for heuristic(default), 2 for hybrid\n"
		"	-i input_file				: input file name e.g. graph.mtx\n"
		"	-s intial_clique_size		: intial max clique size e.g. default 0\n"
		"   -u upper_bound_clique_size  : upper bound on clique size e.g. default vertex count\n"
		"	-p							: This parameter if want to print the max clique\n";
	fprintf(stderr, params, argv0);
	exit(-1);
}

int getDegree(vector<long long>* ptrVtx, int idx)
{
	return ( (*ptrVtx)[idx+1] - (*ptrVtx)[idx] );
}

