/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*   Files: main_fmc.cpp, findClique.cpp, findCliqueHeu.cpp, utils.cpp		  */
/*   			findClique.h, findCliqueHeuOld.cpp, Makefile              		  */
/*                                                                           */                                                                        
/*   Description:  a library for finding the maximum clique from a graph     */                                                   
/*                                                                           */
/*                                                                           */
/*   Authors:  Md. Mostofa Ali Patwary and Bharath Pattabiraman              */
/*            EECS Department, Northwestern University                       */
/*            email: {mpatwary,bpa342}@eecs.northwestern.edu                 */
/*                                                                           */
/*   Copyright, 2012, Md. Mostofa Ali Patwary and Bharath Pattabiraman       */
/*   See COPYRIGHT notice in top-level directory.                            */
/*                                                                           */
/*   Please site the following publication if you use this package           */
/*	  Bharath Pattabiraman, Md. Mostofa Ali Patwary, Assefaw H. Gebremedhin2, */
/*	  Wei-keng Liao, and Alok Choudhary.												  */
/*	  "A Hierarchical Pruning Algorithm for											  */ 
/*	  the Maximum Clique Problem on Large Graphs"									  */
/*   submitted to ALENEX'13 																  */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "findClique.h"

int iMaxDegree, iNotComputed, iMaxClique;

int main(int argc, char *argv[])
{
	int opt, algType, orderType;
	string strDir, strInput, strOutput;
	int print_clique = 0, threads = 1;
	iMaxClique = 0;
	int upper_bound = -1;

	while ((opt=getopt(argc,argv,"u:x:t:i:o:s:p"))!= EOF)
	{
		switch (opt)
		{
                        case 'u':
				upper_bound = atoi(optarg);
				break;
			case 't':
				algType = atoi(optarg);
				break;
			case 'x':
				threads = atoi(optarg);
				break;
			case 's':
				iMaxClique = atoi(optarg);
				break;
			case 'i':
				strInput = optarg;
				break;
			case 'p':
				print_clique = 1;
				break;
			case '?':
				usage(argv[0]);
				break;
			default:
				usage(argv[0]);
				break;
		}
	}

	if(!fexists(strInput.c_str()))
	{
		usage(argv[0]);
		exit(-1);
	}
	FILE* fpInputFileNames = fopen(strInput.c_str(), "r+t");
	FILE* fpOutputFile = fopen(strOutput.c_str(), "a+t");

	if(fpInputFileNames == NULL)
	{
		usage(argv[0]);
		exit(-1);
	}

	cout << "\n\nFile Name ------------------------ " << strInput.c_str() << endl;
	if(!fexists(strInput.c_str()) )
	{
		cout << "File not found!" << endl;
		return 1;
	}

	omp_set_num_threads(threads);

	double seconds = wtime();
	CGraphIO gio;
	gio.readGraph(strInput);

	cout << "Reading time " << wtime() - seconds << endl;
	iMaxDegree = gio.GetMaximumVertexDegree();
	cout << "Vertices " << gio.GetVertexCount() << " Edges " << gio.GetEdgeCount() << " Max degree " << iMaxDegree << endl;

	if(upper_bound == -1)
	{
		cout << "Computing DEFAULT upper bound ";
		seconds = wtime();
		//upper_bound = gio.GetVertexCount();
    		gio.ComputeSmallestLastOrdering();
    		upper_bound = gio.MaximumBackDegree();
		cout << " time " << wtime() - seconds << " value " << upper_bound << endl;
	}
	else
		cout << "Given upper bound " << upper_bound << endl;




	vector <int> max_clique_data;
	max_clique_data.reserve(gio.GetVertexCount());

	seconds = wtime();

	switch (algType)
	{
		case 0:
			//Exact algorithm with lower bound 0.
			cout << "Running Exact Algorithm ... " << endl;
			iMaxClique = maxClique(gio, iMaxClique, iNotComputed, max_clique_data, upper_bound);
			break;
		case 1:
			//Heuristic
			cout << "Running Heuristic ... " << endl;
			//iMaxClique = maxCliqueHeuOld(gio, iNotComputed);
			iMaxClique = maxCliqueHeu(gio, iNotComputed, max_clique_data, upper_bound);
			break;
		case 2:
			//Pass result of heuristic as lower bound to exact algo.
			cout << "Running Hybrid Algorithm ... " << endl;
			//Has to be changed to the new version once bugs have been fixed.
			//iMaxClique = maxCliqueHeuOld(gio, iNotComputed);
			iMaxClique = maxCliqueHeu(gio, iNotComputed, max_clique_data, upper_bound);
			cout << "Heuristic found cliqe of size " << iMaxClique << endl;
			iMaxClique = maxClique(gio, iMaxClique, iNotComputed, max_clique_data, upper_bound);
			break;
		default:
			//Heuristic
			//iMaxClique = maxCliqueHeuOld(gio, iNotComputed);
			iMaxClique = maxCliqueHeu(gio, iNotComputed, max_clique_data, upper_bound);
			cout << "Running Heuristic by default ... " << endl;
			break;
	}

	seconds = wtime() - seconds;

	cout << "Max clique : " << iMaxClique << endl;
	cout << "Time taken : " << seconds << " SEC" << endl;			

	fclose(fpInputFileNames);

	if(print_clique == 1)
		print_max_clique(max_clique_data);		

	max_clique_data.clear();

	return 0;
}
