function export2mtx(data, prefix)
% export graph to symmetric matrix market file format
%   - edges should only be specified in the lower triangular
%
%   ------------
%   %%MatrixMarket matrix coordinate pattern symmetric
%   8 8 6
%   3 2
%   . .
% 
% see http://math.nist.gov/MatrixMarket/formats.html#MMformat
% 
% Ryan Rossi
% Copyright 2012


setup_paths

nargin
if nargin == 0,
    fprintf('...\n')
    data = 'graphs';
end

if nargin < 2,
   prefix = ''; 
end

% load the list of graphs
graphlist = get_graphlist(data);



% load graph, and export
gdata = containers.Map();
for i=1:size(graphlist,1)
    ginfo = struct();
    A = load_graph(graphlist{i,2});
    ginfo.name = graphlist{i,2};
    assert(all(diag(A)==0));
    ginfo.nverts = size(A,1);
    ginfo.nedges = nnz(A)/2;
    ginfo.degrees = full(sum(A,2));
    ginfo
    
    write_mtx(A,[prefix,ginfo.name,'.mtx']);
    
end


