function gdata = heur_accuracy(gdata, graphname)
% Computes max clique recall from heuristic,
%   and adds this measure to gdata
%
% Heuristic is strictly lower than the exact, and thus
%   we compute accuracy taking into account:
%       1. size of the max clique from heur
%       2. vertices in max clique ^^
%
% Ryan Rossi, Purdue University
% Copyright 2012
%

for graph=gdata.keys,
    clear mc cores maxcore mk_verts mc_verts common_verts gamma_K verts gamma_H 
    info = gdata(graph{1});
    
    mc = info.mc_size;
    mc_verts = info.mc_verts;
    
    verts = info.mc_heur_verts;
    common_verts = intersect(mc_verts,verts);
    
    if ~isempty(mc_verts) && ~isempty(verts),
        assert(length(mc_verts) == mc)
        
        gamma_H = length(common_verts) / mc;
        info.gamma_H = gamma_H;
        
        fprintf('%s, gamma_H=%0.2f, %d verts in common [mc=%d, mc_heur=%d] \n', ...
            info.name, info.gamma_H, length(common_verts), length(mc_verts), length(verts));
    else
        info.gamma_H = -1; %no exact
    end
    gdata(graph{1}) = info;
end

if nargin > 1,
    save(['../results/',graphname,'/gdata'], 'gdata', '-v7.3');
end