function gdata = kcore_recall(gdata, graphname)
% Computes max clique recall from max kcore, 
%   and adds this measure to gdata
%
% Ryan Rossi, Purdue University
% Copyright 2012
%

cont = 0;
graph_count = 0;
for graph=gdata.keys,
    clear mc cores maxcore mk_verts mc_verts common_verts gamma_K
    info = gdata(graph{1});
    
    cores = info.cores;
    maxcore = max(cores);
    mk_verts = find(cores == maxcore); 
    
    mc = info.mc_size;
    mc_verts = info.mc_verts;
    
    common_verts = intersect(mc_verts,mk_verts);
  
    if ~isempty(mc_verts),
        assert(length(mc_verts) == mc)
        assert(maxcore+1 >= mc)
        
        gamma_K = length(common_verts) / mc; 
        info.gamma_K = gamma_K;
        
        if length(common_verts) == mc, cont = cont + 1; end;
        graph_count = graph_count + 1;
        
        fprintf('%s, gamma_K=%0.2f, %d mc verts in the kcore [max_kcore=%d, mc=%d] (size of max_kcore=%d) \n', ...
            info.name, info.gamma_K, length(common_verts), maxcore, length(mc_verts), length(mk_verts));
    else
        info.gamma_K = -1;
    end
    gdata(graph{1}) = info;
end
fprintf('%d graphs where maxclique is contained completely in kcore, out of %d graphs \n', cont, graph_count);

if nargin > 1,
    save(['../results/',graphname,'/gdata'], 'gdata', '-v7.3');
end