function print_bounds(data,style)
% prints the 'chain' of bounds for max clique
%
% Ryan Rossi, Purdue University
% Copyright 2012
%

setup_paths

if nargin < 2,
    style = 'compact';
end

if nargin == 0,
    data = 'graphs'
end
graphlist = get_graphlist(data);

%update_maxclique(data)
%update_clique_verts(data)

update_mc(data)

dpath = ['../results/',data];
load([dpath,'/gdata'])
%gdata = update_types(gdata,data);
%gdata = update_names(gdata,data);

gdata = kcore_recall(gdata, data);
gdata = heur_accuracy(gdata, data);

sgdata = cont2struct(gdata)

header = {
    '\begin{table*}[h!]';
    '\caption{Network statistics of the static graphs, including the size of the maximum clique and the upperbounds. Here, $d_{\max}$ and $d_{\text{avg}}$ are the largest and average degrees, respectively.  Also, $\bar{\kappa}$ is the mean clustering coefficient; $T$ and $T_{\text{avg}}$ are the maximum number and average number of triangles incident on a vertex; $K$ is the largest degree for a $k$-core to exist, $\omega$ is the clique number, and $\Gamma_K = \frac{|C \cap K|}{|C|}$ is the recall measurement by using the $k$-core to find the maximum clique.}';
    '\vspace{1mm}';
    '\label{table:mc-data}';
    '\centering\small\scriptsize';
    '\begin{tabularx}{\linewidth}{ r XXXXXXXXXXX X XXX}';
    '\toprule';
    '\textbf{Graph} & $|V|$ & $|E|$ & ';
    '$d_{\text{avg}}$ & $\mathbf{\bar{\kappa}}$ & $T$ & $T_{\text{avg}}$ & '
    '$\tilde{\omega}$ & $\omega$ & $K$ & $\sqrt{2T}$ & $d_{\max}$ & $\Gamma_H$ & $\Gamma_K$ \\'
    '\midrule';
    };
for i=1:length(header), fprintf('%s\n',header{i}); end;

max_strsize = 12;
types = unique({sgdata.type});
for ti=1:numel(types)
    type = types{ti};
    tgraphs = sgdata(strcmp({sgdata.type},type));
    %[~,p] = sort([tgraphs.nverts]);
    [~,p] = sort([tgraphs.mc_size]);
    for gi=1:numel(p)
        clear g maxcore maxclique tri_ub mc_heur_size gamma_K gamma_H
        suffix = ''; prefix = '';
        
        g = tgraphs(p(gi));
        maxcore = g.maxcore;
        maxclique = g.mc_size;
        tri_ub = ceil(sqrt(2*max(g.triangles)));
        g.tri_bound = tri_ub; %todo: eventually remove this update, reflected in computations
        
        mc_heur_size = g.mc_heur_size;
        
        gamma_K = g.gamma_K;
        gamma_H = g.gamma_H;
        
        name = rm_type(lower(strip(g.name)));
        name = substr(name,1,max_strsize);
        
        check_bounds(g)
        
        fprintf('   \\textsc{%s} & ',name);
        if strcmp(style,'compact'),
            fprintf('%s & %s & ',...
                abbr_num(g.nverts), abbr_num(g.nedges));
        else
            fprintf('%d & %d & ',...
                g.nverts, g.nedges);
        end
        
        fprintf('%.1f & %.2f & ',...
            mean(g.degrees), mean(g.clustercoef));
        
        if strcmp(style,'compact'),
            fprintf('%s & ',abbr_num(max(g.triangles)));
        else
            fprintf('%d & ',max(g.triangles));
        end
        fprintf('%.1f ',mean(g.triangles));
        

        %heuristic maxclique
        prefix = ''; suffix = '';
        if mc_heur_size == maxclique,
            prefix = '\cellcolor{lightgreen}';
        end
        fprintf('& %s %.0f%s ',prefix,mc_heur_size,suffix);
        
        %MAX CLIQUE
        fprintf('& \\textbf{%d} ',maxclique);
        
        %max kcore upperbounds
        prefix = ''; suffix = '';
        if maxcore+1 == maxclique,
            prefix = '\cellcolor{lightblue}';
        end
        fprintf('& %s %d%s ',prefix,maxcore,suffix);

        %triangle upperbounds
        prefix = ''; suffix = '';
        if tri_ub+1 == maxclique,
            prefix = '\cellcolor{lightyellow}';
        end
        if (tri_ub+1) <= (maxcore+1),
            suffix = '$^{\star}$';
        end
        fprintf('& %s %.0f%s ',prefix,tri_ub,suffix);
        
        fprintf('& %.0f ',max(g.degrees));
        
        
        %============
        %  RECALL
        %============
        %recall of maxclique using heuristic
        prefix = ''; suffix = '';
        if gamma_H == 1,
            prefix = '\cellcolor{lightred}';
        end
        if gamma_H == -1, %exact solution not computed
            fprintf('& - ');
        else
            fprintf('& %s %.1f%s ',prefix,gamma_H,suffix);
        end
        
        %recall of maxclique using K
        prefix = ''; suffix = '';
        if gamma_K == 1,
            prefix = '\cellcolor{lightred}';
        end
        if gamma_K == -1, %exact solution not computed
            fprintf('& - ');
        else
            fprintf('& %s %.1f%s ',prefix,gamma_K,suffix);
        end
        
        fprintf('\\\\ \n');
    end
    fprintf('\\midrule \n');
end
fprintf('\\end{tabularx} \n\\end{table*} \n');
