function print_mc_table(data)
% prints the stats, mc, and upperbounds to table
%
% Ryan Rossi, Purdue University
% Copyright 2012
%

setup_paths

if nargin == 0, data = 'graphs'; end;
graphlist = get_graphlist(data);

update_mc(data)

dpath = ['../results/',data];
load([dpath,'/gdata'])
%gdata = update_types(gdata,data);
%gdata = update_names(gdata,data);

gdata = kcore_recall(gdata, data);
gdata = heur_accuracy(gdata, data);

sgdata = cont2struct(gdata);

header = {
    '\begin{table*}[h!]';
    '\caption{Properties of the graphs, including the size of the max clique and corresponding upperbounds.}';
    '\vspace{1mm}';
    '\label{table:mc-data}';
    '\centering\small\scriptsize';
    '\begin{tabularx}{\linewidth}{ r XXXXXXXXXXX cc}';
    '\toprule';
    '\textbf{Graph} & $|V|$ & $|E|$ & ';
    '$d_{\max}$ & $d_{\text{avg}}$ & $\mathbf{\bar{\kappa}}$ & '
    '$T$ & $T_{\text{avg}}$ & $\sqrt{2T}$ & $K$ & $\omega$ &  $\Gamma_K$\\'
    '\midrule';
    };

for i=1:length(header),
    fprintf('%s\n',header{i});
end

max_strsize = 12;
print_ginfo = @(g) ...
    fprintf('   \\textsc{%s} & %d & %d & %d & %.1f & %.2f & %d & %.1f & %.0f & %d & \\textbf{%d} & %.2f \\\\ \n', ...
    strip(lower(strrep(strrep(strrep(substr(g.name,1,max_strsize), '_', '-'),'2009',''),'rt-', '\scriptsize \#'))),...
    g.nverts, g.nedges,...
    max(g.degrees), mean(g.degrees), mean(g.clustercoef), ...
    max(g.triangles), mean(g.triangles), sqrt(2*max(g.triangles)), ...
    g.maxcore, g.mc_size, g.gamma_K);


types = unique({sgdata.type});
for ti=1:numel(types)
    type = types{ti};
    tgraphs = sgdata(strcmp({sgdata.type},type));
    [~,p] = sort([tgraphs.mc_size]);
    for gi=1:numel(p)
        print_ginfo(tgraphs(p(gi)));
    end
    fprintf('\\midrule \n');
end
fprintf('\\end{tabularx} \n\\end{table*} \n');
