'''
@author: Ryan Rossi

python2.6 merge_info.py <mc,tscc> <folder: graphs, retweets> <maxlen: dataname>

inputs:
1. <mc,tscc>: 	problem, used mainly for indicating the path when searching for info
2. folder: 		str indicating the folder to search when merging info, for instance, '../mc/results/fmc/<folder>'
		   		this must match the previous folder used in start_mc.py

Example
---------
python2.6 merge_info.py mc graphs

'''

import sys
import os
from sets import Set
import operator
from operator import itemgetter, attrgetter


def uniqueLists(g,name):
    unique = {}
    for g in graphs:
        #if len(unique) > 0:
        if name in g[2]:
            m = unique.get(g[1],'f')
            if m == "f":    
                print g[1] + ' added!'
                unique[g[1]] = g
            elif m[4] < g[4]:
                print g[1] + ' switched!'
                #rm prev
                unique[m[1]] = m
    return unique


comp = 'mc'
if len(sys.argv[1]) > 1:
	comp = sys.argv[1]

if len(sys.argv[2]) > 2:
    folder = sys.argv[2]

path = '../' + comp + '/results/fmc/' + folder + '/'
dirList=os.listdir(path)

maxlen = 12
if len(sys.argv) > 3:
    maxlen = int(sys.argv[3])

if len(sys.argv) > 4:
    dirList = [i for i in dirList if sys.argv[4] in i]
if len(sys.argv) > 5: #not
    dirList = [i for i in dirList if sys.argv[5] not in i]


print dirList
edge_list = []
vertices = Set([])
fn_list = [];
sh_list = [];
problem = ['tscc','cc']

graphs = []
tscc_bool = 0
tcc_bool = 0
mc_bool = 0
mc = 0
for fname in dirList:
    stats_list = []
    stats_heur = {}
    stats_exact = {}
    if len(fname.split('.')[0]) > 0 and '.DS' not in fname:
        line_list = open(path + fname, 'r').readlines()
        if "scc" in fname and not "static_scc_" in fname:
            stats_list.append('tscc')
            tscc_bool = tscc_bool + 1
            tscc = 'tscc'
            out_str = 'tscc'
        elif "cc" in fname and not "static_scc_" in fname:
            stats_list.append('tcc')
            tcc_bool = tcc_bool + 1
            tcc = 'tcc'
            out_str = 'tcc'
        else:
            stats_list.append('mc')
            mc_bool = mc_bool + 1
            mc = 'mc'
            out_str = 'mc'
        
        for line in line_list:
            if "File Name" in line:
                fileparts = line.split('/')
                name = fileparts[len(fileparts)-1].split('.')[0]
                stats_list.append(name)
            
            if "Running" in line:
                if "Exact" in line:
                    stats_list.append('exact')
                elif "Heuristic" in line:
                    stats_list.append('heur')
                
            if "Max clique" in line:
                stats_list.append(int(line.split(':')[1].replace('\n','').strip()))
                
            if "Time taken" in line:
                x = float(line.split(':')[1].replace('SEC','').replace('\n','').strip())
                x_str = format(x, '.2f').rstrip('0').rstrip('.')
                if x_str == '0':
                    x_str = '$< 0.01$'
                stats_list.append(x_str)
                
                
            if "The maximum clique" in line:
                stats_list.append(line.split(':')[1].replace('\n','').strip())
                
    if len(stats_list) > 5:
        graphs.append(stats_list)


graphs_heur = {}
graphs_exact = {}
for g in graphs:
    print g
    if "heur" in g[2]:
        graphs_heur[g[1]] = g
    elif "exact" in g[2]:
        graphs_exact[g[1]] = g


H = uniqueLists(g,"heur")
E = uniqueLists(g,"exact")
print 'h=' + str(len(H)) + ', e=' + str(len(E)) + '\n'


fout_verts = open(path + '/info_' + folder.replace('/','') + '_verts_' + out_str + '.txt', 'w+')
for g in E:
    s = ''
    for x in E[g]:
        s = s + str(x) + '\t'
    fout_verts.write(s + '\n')
for g in H:
    s = ''
    for x in H[g]:
        s = s + str(x) + '\t'
    fout_verts.write(s + '\n')
fout_verts.close()

fout = open(path + '/info_' + out_str + '.txt', 'w+')
for g in E:
    d = E[g]
    s = str(d[0]) + ',' + str(d[2]) + ',' + str(d[1]) + ',' + str(d[3]) + ',' + str(d[4]) + ',' + str(d[5])
    fout.write(s + '\n')
for g in H:
    print g
    d = H[g]
    s = str(d[0]) + ',' + str(d[2]) + ',' + str(d[1]) + ',' + str(d[3]) + ',' + str(d[4]) + ',' + str(d[5])
    fout.write(s + '\n')
fout.close()


fout = open(path + '/info_' + folder.replace('/','') + '_' + out_str + '.txt', 'w+')
for g in E:
    d = E[g]
    s = str(d[0]) + ',' + str(d[2]) + ',' + str(d[1]) + ',' + str(d[3])
    fout.write(s + '\n')
for g in H:
    print g
    d = H[g]
    s = str(d[0]) + ',' + str(d[2]) + ',' + str(d[1]) + ',' + str(d[3])
    fout.write(s + '\n')
fout.close()


#======================
# Save file
#======================
fout = open(path + '/table_' + folder.replace('/','') + '_' + out_str + '.txt', 'w+')

fout.write('\\begin{table}[t!]\n')
if "scc" in out_str:
    fout.write('\\caption{\\textbf{Largest Temporal Strongly Connected Component}. }\n')
elif "cc" in out_str:
    fout.write('\\caption{\\textbf{Largest Temporal Connected Component}. }\n')
else:
    fout.write('\\caption{\\textbf{Max Cliques in Real Data}. }\n')
    
fout.write('\\vspace{1mm}\n')
fout.write('\\label{table:mc_data}\n')
fout.write('\\centering\\small\n')
fout.write('\\begin{tabular}{l !{\\vrule width 0.1mm} cc!{\\vrule width 0.3mm}rr}\n')
if "scc" in out_str:
    fout.write('\\TT  &\\multicolumn{2}{c!{\\vrule width 0.3mm}}{\\small \\textbf{TSCC Size}} & \\multicolumn{2}{c}{\\small\\textbf{Time} (seconds)}\\\\ \n')
elif "cc" in out_str:
    fout.write('\\TT  &\\multicolumn{2}{c!{\\vrule width 0.3mm}}{\\small \\textbf{TWCC Size}} & \\multicolumn{2}{c}{\\small\\textbf{Time} (seconds)}\\\\ \n')
else:
    fout.write('\\TT  &\\multicolumn{2}{c!{\\vrule width 0.3mm}}{\\small \\textbf{Max Clique Size}} & \\multicolumn{2}{c}{\\small\\textbf{Time} (seconds)}\\\\ \n')
                
fout.write('\\cline{2-5}\n')
fout.write('\\TT \\textbf{Graph} &  \\textsf{\\textbf{Exact}}  & \\textsf{\\textbf{Heur}} & \\textsf{\\textbf{Exact}} & \\textsf{\\textbf{Heur}} \\\\ \n')
fout.write('\\noalign{\\hrule height 0.3mm}\n')
fout.write('\\TT\n')


types = {}
name2type = {}

if 'retweets' in folder:
    fn = '../' + comp + '/code/dynamic_retweets.m'
else:
    if "mc" in out_str or "static_scc_" in out_str:
        fn = '../' + comp + '/code/graphs.m'
    else:
        fn = '../' + comp + '/code/dynamic_graphs.m'
    
fin = open(fn, 'r')
for line in fin:
    if 'graphlist' not in line and '}' not in line and '%' not in line:
        if len(line) > 1:
            line = line.split(',')
            name = line[1].rstrip('\'').lstrip('\'').replace(';','').replace('\n','').replace('\'','')
            type = line[3].rstrip('\'').lstrip('\'').replace(';','').replace('\n','').replace('\'','')
            key = types.get(type,'f')
            if 'f' in key:
                types[type] = [name]
            else:
                key.append(name)
                types[type] = key
                
            name2type[name] = type

types_list = types.keys()
types_sorted = sorted(types_list)
print types_sorted
print '\n'

ct = 0
max_types = len(types_sorted)
for t in types_sorted:
    datasets = types[t] #set of data for type t
    H_type = []
    E_type = []
    for d in datasets:
        h = H.get(d, 'f')
        if 'f' not in h:
            H_type.append(h)
        e = E.get(d, 'f')
        if 'f' not in e:
            E_type.append(E[d])
            
    if len(E_type) >= len(H_type):
            E_sorted = sorted(E_type, key=itemgetter(3))
            print E_sorted
            max = len(E_sorted)
            c = 0
            for g in E_sorted:
                isrt = 0
                if g[3] > 0:
                    name = ''
                    heur = H.get(g[1], "?")
                    if "?" == heur:
                        heur_size = "?"
                        heur_time = "?"
                    else:
                        heur_size = heur[3]
                        heur_time = heur[4]
         
                    name = g[1]
                    print name
                    
                    if 'rt_' in name:
                        isrt = 1
                    name = name.replace('_','-')
                    name = name.lower()
                    name = name[0:maxlen]
                    
                    if isrt == 1:
                        name = '\\scriptsize \\#' + name
                    
                    line = '   \\textsc{' + name + '} & ' + str(g[3]) + ' & ' + str(heur_size) + ' & ' + str(g[4]) + '  & ' + heur_time + '\\\\ \n'
                    fout.write(line)
                    c = c + 1
    else:
            H_sorted = sorted(H_type, key=itemgetter(3))
            print H_sorted
            max = len(H_sorted)
            c = 0
            for g in H_sorted:
                isrt = 0
                if g[3] > 0:
                    c = c + 1
                    heur = E.get(g[1], "?")
                    if "?" == heur:
                        exact_size = "?"
                        exact_time = "?"
                    else:
                        exact_size = heur[3]
                        exact_time = heur[4]
                    name = g[1]
                    if 'rt_' in name:
                        isrt = 1
                    name = name.replace('_','-')
                    name = name.lower()
                    name = name[0:maxlen]
                    
                    if isrt == 1:
                        name = '\\scriptsize \\#' + name
                    line = '   \\textsc{' + name + '} & ' + str(exact_size) + ' & ' + str(g[3])+ ' & ' + exact_time + '  & ' + str(g[4]) + '\\\\ \n'
                    fout.write(line)
    
    if ct < max_types-1:
        fout.write('\\cline{1-5}\n\\TT\n')
    ct = ct + 1
fout.write('\\bottomrule\n')
fout.write('\\end{tabular}\n')
fout.write('\\end{table}\n')

fout.close()