'''
@author: Ryan Rossi

python2.6 start_mc.py <mc,tscc> <threads> <folder: graphs, retweets> <alg:0=exact,1=heur,2=both> <substr> <not substr> <ub>
Creates sh files for each graph, and finds the max clique of each graph, one at a time.

Example:
-----------
python2.6 start_mc.py tscc 4 graphs 2

'''

import sys
import os
from sets import Set
import errno


def ensure_path_exists(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise

comp = 'mc'
if len(sys.argv) > 1:
	comp = sys.argv[1]

threads = 1;
if len(sys.argv) > 2:
    threads = sys.argv[2]

if len(sys.argv) > 3:
    folder = sys.argv[3]

algorithms = []
if len(sys.argv) > 4:
    al = int(sys.argv[4])
    if al == 0:
        algorithms = ['exact']
    elif al==1:
        algorithms = ['heur']
    else: #2
        algorithms = ['exact', 'heur']


path="../" + comp + "/data/output/"
#path="../../data/output/"  # insert the path to the directory of interest here
dirList=os.listdir(path)
if len(sys.argv) > 5: #str in filename
    dirList = [i for i in dirList if sys.argv[5] in i]
if len(sys.argv) > 6: #str not in filename
    dirList = [i for i in dirList if sys.argv[6] not in i]

#todo: allow list of ub for a set of graphs
ub = 0
if len(sys.argv) > 7:
    ub = int(sys.argv[7])

print algorithms
print dirList
edge_list = []
vertices = Set([])
fn_list = [];
sh_list = [];
problem = ['tscc','cc']

dir_str = 'tmp'
comp_path = '../' + comp + '/scripts/' + dir_str + '/'
ensure_path_exists(comp_path)
for fname in dirList:
    print fname
    if len(fname.split('.')[0]) > 0 and '.DS' not in fname and '.mtx' in fname:
        fn_list.append(fname)
        if 'scc' in fname:
            prob = 'tscc_'
        elif 'cc' in fname:
            prob = 'tcc_'
        else:
            prob = ''
        a = 0
        for alg in algorithms:
            #shfile = prob + alg + '_' + fname.split('.')[0] + '.sh'
			shfile = prob + alg + '_' + fname.split('.')[0] + '.sh'
			fout = open(comp_path + shfile, "w+")
			fout.write('#!/bin/bash\n')
			fout.write('#\n # Compute MC/TSCC \n')
			fout.write('# nohup sh ./tscc.sh >& results_tscc.txt  &')
			fout.write('#\n#\n\n\n')
			if "exact" in alg:
				a = 0
			elif "heur" in alg:
				a = 1
            
			fout.write('START=$(date +%s)\n\n')
			fout.write('graphs[1]=' + fname + '\n\n')

			if ub == 0:
				#fout.write('./../../../fastMaxClique/src_parallel/fmc_par -t ' + str(a) + ' -p -i ' + path + fname + ' -x ' + str(threads) + '\n\n')
				fout.write('./../fastMaxClique/src_parallel/fmc_par -t ' + str(a) + ' -p -i ' + path + fname + ' -x ' + str(threads) + '\n\n')
			else:
				#fout.write('./../../../fastMaxClique/src_parallel/fmc_par -t ' + str(a) + ' -p -i ' + path + fname + ' -x ' + str(threads) + ' -u ' + str(ub) + '\n\n')
				fout.write('./../fastMaxClique/src_parallel/fmc_par -t ' + str(a) + ' -p -i ' + path + fname + ' -x ' + str(threads) + ' -u ' + str(ub) + '\n\n')

			fout.write('END=$(date +%s)\n')
			fout.write('DIFF=$(( $END - $START ))\n')
			fout.write('echo \"It took $DIFF seconds\"\n')
			fout.close()
			sh_list.append(shfile)


results_path = '../' + comp + '/results/fmc/' + folder + '/'
shexe = comp_path + 'run_all.sh'
fout = open( shexe, "w+" )
fout.write('#!/bin/bash\n')
fout.write('#\n # Compute MC/TSCC \n')
fout.write('#\n#\n')
fout.write('# nohup sh ./run_all.sh >& results_run_all.txt  &')
fout.write('#\n#\n')
fout.write('\n\n')
for file in sh_list:
	print file
	fout.write('sh ./' + comp_path + file + ' >& ' + results_path + 'results_' + file.split('.')[0] + '.txt \n')
fout.close()


ensure_path_exists(results_path)
ret = os.system('nohup sh ./' + shexe + ' >& ' + results_path + '/results_run_all.txt  &')
